package com.cezar.cezargis.ws.zbdbdot;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;

import org.apache.log4j.Logger;
import org.postgresql.util.Base64;

import com.cezar.cezargis.ws.zbdbdot.model.ExportModel;
import com.cezar.cezargis.ws.zbdbdot.service.DbService;
import java.math.BigInteger;

public class Export {

    private static final Logger logger = Logger.getLogger(Export.class);

    /**
     * Przygotowanie pliku do exportu.
     * @throws ZbdbdotWebServiceException 
     * 
     */
    public static void runBackup() throws ZbdBdotWebServiceException {

	ExportModel em = getExportObject();
	// Kiedy nie mam aktualnie tworzonego pliku exportu
	if (em == null) {
	    Date date = new Date();
	    final String fileName = DbService.FILE_DUMP + date.getTime() + ".backup";
	    long startCountingTime = System.currentTimeMillis();

	    final String pgDump = "\"" + DbService.PG_DUMP + "\"" + " --host "
		    + DbService.DB_HOST + " --port " + DbService.DB_PORT
		    + " --username " + DbService.DB_USER + " --format custom"
		    + " --verbose" + " --file " + "\"" + fileName + "\""
		    + " --schema " + DbService.DB_SCHEMA_DUMP + " "
		    + DbService.BD_BASE;
	    // Zapis do bazy przed tworzeniem dumpa
	    int saveToDB = DbService.saveToDB(fileName, startCountingTime,
		    FileStatus.AKTUALNY, ExportStatus.GENEROWANY);

	    //pobranie z bazy danych aktualniego obiektu z statusem aktualny
	    final ExportModel emNew = getExportObject();

	    if (emNew != null) {
		// Zapis logowania
		DbService.setToLog(emNew.getZbdtId(), ExportStatus.GENEROWANY);

		//Rozpoczęcie procesu dump
		new Thread(new Runnable() {

		    @Override
		    public void run() {
			try {
			    String line = null;
			    String error = null;
			    // Poprawny zapis w bazie - tworzenie dumpa
			    Process process = Runtime.getRuntime().exec(pgDump);

			    BufferedReader stdError = new BufferedReader(
				    new InputStreamReader(process.getErrorStream()));

			    // Przełapanie błedów
			    while ((line = stdError.readLine()) != null) {
				error = (error == null ? line : error + "\n" + line);
				logger.info(line);
			    }

			    int waitFor = process.waitFor();
			    if (waitFor == 0) {
				// Zapis do bazy poprawnego dumpa
				long fileSize = getFileSize(fileName);
				String md5 = getChecksumMD5(fileName);
				emNew.setZbdtPlikExportStatus(ExportStatus.WYGENEROWANY);
				emNew.setZbdtPlikSumakontrolna(md5);
				emNew.setZbdtPlikRozmiar(fileSize);
				DbService.updateFileExportObject(emNew);
				DbService.setToLog(emNew.getZbdtId(), ExportStatus.WYGENEROWANY);
				// Wyczyszczenie schematu
				DbService.truncateSchema();
			    } else {
				// BŁAD
				emNew.setZbdtPlikExportStatus(ExportStatus.BLAD_GENEROWANIA);
				emNew.setZbdtPlikStatus(FileStatus.NIEAKTUALNY);
				DbService.updateFileExportObject(emNew);
				DbService.setToLog(emNew.getZbdtId(), ExportStatus.BLAD_GENEROWANIA);
			    }
			} catch (ZbdBdotWebServiceException ex) {
			    try {
				emNew.setZbdtPlikStatus(FileStatus.NIEAKTUALNY);
				saveBdChanges(emNew, ExportStatus.BLAD_GENEROWANIA);
			    } catch (ZbdBdotWebServiceException db) {
				db.printStackTrace();
			    }
			} catch (IOException e) {
			    emNew.setZbdtPlikStatus(FileStatus.NIEAKTUALNY);
			    try {
				saveBdChanges(emNew, ExportStatus.BLAD_GENEROWANIA);
			    } catch (ZbdBdotWebServiceException db) {
				db.printStackTrace();
			    }
			    e.printStackTrace();
			} catch (InterruptedException e) {
			    // Bład zapisu w bazie
			    e.printStackTrace();
			}
		    }
		}).start();
		//throw new ZbdBdotWebServiceException(ExportStatus.GENEROWANY);
	    }
	} else {
	    // plik aktualnie tworzony
	    //throw new ZbdBdotWebServiceException(ExportStatus.GENEROWANY);
	}
    }

    /**
     * Pobranie rozmiaru przygotowywanego pliku exportu.
     * 
     * @return Long
     */
    public static Long getFileSizeFromDB() throws ZbdBdotWebServiceException {
	ExportModel em = getExportObject();
	if (em != null) {
	    return em.getZbdtPlikRozmiar();
	}
	return null;
    }

    /**
     * Pobranie pliku
     * 
     * @return
     * @throws Exception
     */
    public static DataHandler getDumpFile() throws ZbdBdotWebServiceException {
	ExportModel em = getExportObject();
	if (em != null) {
	    // Sprawdzenie statusu generowania
	    ExportStatus es = em.getZbdtPlikExportStatus();

	    if (ExportStatus.WYGENEROWANY.equals(es)
		    || ExportStatus.ROZPOCZECIE_WYSYLANIA.equals(es)) {
		File file = new File(em.getZbdtPlikPath());
		if (file.exists()) {
		    // zmiana statusu na StatusWysylania
		    em.setZbdtPlikExportStatus(ExportStatus.ROZPOCZECIE_WYSYLANIA);
		    DbService.updateFileExportObject(em);
		    DbService.setToLog(em.getZbdtId(), ExportStatus.ROZPOCZECIE_WYSYLANIA);
		    return new DataHandler(new FileDataSource(file));
		} else {
		    throw new ZbdBdotWebServiceException(ExportStatus.BLAD);
		}
	    } else if (ExportStatus.GENEROWANY.equals(es)) {
		// Sprawdzenie statusu pliku
		if (checkForGeneratingFileTime(em.getZbdtPlikStart())) {
		    // Plik generowany przekracza okreslony czas geenrowania -
		    // rzadanie pobrania nowego pliku
		    em.setZbdtPlikStatus(FileStatus.NIEAKTUALNY);
		    DbService.updateFileExportObject(em);
		    runBackup();
		    throw new ZbdBdotWebServiceException(ExportStatus.GENEROWANY);
		} else {
		    throw new ZbdBdotWebServiceException(ExportStatus.GENEROWANY);
		}
	    } else {
		em.setZbdtPlikStatus(FileStatus.NIEAKTUALNY);
		DbService.updateFileExportObject(em);
		throw new ZbdBdotWebServiceException(ExportStatus.BLAD);
	    }
	}
	throw new ZbdBdotWebServiceException(ExportStatus.BLAD);
    }

    /**
     * Potwierdzenia prawidłowego odebrania pliku
     * 
     * @param FileSieze
     *            - rozmiar pliku
     * @param checksumMD5
     *            - sumę kontrolną (MD5),
     * @throws ZbdbdotWebServiceException
     */
    public static void checkFile(long FileSieze, String checksumMD5)
	    throws ZbdBdotWebServiceException {
	ExportModel em = getExportObject();
	if (em != null) {
	    if (checksumMD5.equals(em.getZbdtPlikSumakontrolna())
		    && FileSieze == em.getZbdtPlikRozmiar()) {
		// Usunięcie pliku
		deleteFile(em.getZbdtPlikPath());
		logger.info("Plik usunuęty");
		// zmiana statusuPotwierdzenia
		em.setZbdtPlikStatus(FileStatus.NIEAKTUALNY);
		saveBdChanges(em, ExportStatus.POTWIERDZENIE_ODBIORU);
	    } else {
		throw new ZbdBdotWebServiceException("checkFile: " + em.getZbdtPlikRozmiar(), null);
	    }
	} else {
	    throw new ZbdBdotWebServiceException("BLAD", null);
	}
    }

    private static ExportModel getExportObject() {
	ExportModel em = null;
	try {
	    em = DbService.getFileExportObject();
	} catch (ZbdBdotWebServiceException zwse) {
	    if (DbStatus.BRAKDANYCH.equals(zwse.getDbStatus())
		    || DbStatus.BLADZAPISU.equals(zwse.getDbStatus())) {
		return null;
	    } else if (DbStatus.WIELDANYCH.equals(zwse.getDbStatus())) {
		try {
		    DbService.setNonActiveStatus();
		    // rzadanie pobrania kolejnego dumpa ponieważ wystapił bład i status został zamieniony na nieaktywny
		    runBackup();
		} catch (ZbdBdotWebServiceException e) {
		    new ZbdBdotWebServiceException("Bład zmianu statusu na nieaktywny", e);
		}
		return null;
	    } else {
		new ZbdBdotWebServiceException("Bład pobierania obiektu z bazy", zwse);
	    }
	}
	return em;
    }

    private static void saveBdChanges(ExportModel em, ExportStatus es) throws ZbdBdotWebServiceException {
	try {
	    em.setZbdtPlikExportStatus(es);
	    DbService.updateFileExportObject(em);
	    DbService.setToLog(em.getZbdtId(), es);
	} catch (ZbdBdotWebServiceException e) {
	    throw new ZbdBdotWebServiceException(e);
	}
    }

    private static boolean deleteFile(String filePath) {
	File f = new File(filePath);

	if (!f.exists()) {
	    throw new IllegalArgumentException("Brak pliku: " + filePath);
	}

	if (!f.canWrite()) {
	    throw new IllegalArgumentException(" Plik zabezpieczony: "
		    + filePath);
	}

	return f.delete();
    }

    private static long getFileSize(String fileName) {
	File file = new File(fileName);

	if (!file.exists() || !file.isFile()) {
	    System.out.println("Brak Pliku");
	    return -1;
	}
	return file.length();
    }

    private static String getChecksumMD5(String fileName) {
	try {
	    int theByte = 0;
	    MessageDigest md = MessageDigest.getInstance("MD5");
	    BufferedInputStream bis = new BufferedInputStream(
		    new FileInputStream(fileName));

	    while ((theByte = bis.read()) != -1) {
		md.update((byte) theByte);
	    }
	    bis.close();
	    byte[] digest = md.digest();
	    String encodeBytes = Base64.encodeBytes(digest);
	    /* test*/
	    BigInteger bi = new BigInteger(1, digest);
	    System.out.print(String.format("%0" + (digest.length << 1) + "X", bi));

	    return encodeBytes;
	} catch (NoSuchAlgorithmException e) {
	    e.printStackTrace();
	} catch (FileNotFoundException e) {
	    e.printStackTrace();
	} catch (IOException e) {
	    e.printStackTrace();
	}
	return null;
    }

    private static boolean checkForGeneratingFileTime(long startCountingTime) {
	long elapsedCountingTimeMillis = System.currentTimeMillis()
		- startCountingTime;
	float elapsedCountingTimeHour = elapsedCountingTimeMillis
		/ (60 * 60 * 1000F);
	Long accessTime = Long.valueOf(DbService.FILE_PROCES);
	if (elapsedCountingTimeHour > accessTime) {
	    return true;
	} else {
	    return false;
	}
    }
}
