package com.cezar.cezargis.ws.zbdbdot.service;

import com.cezar.cezargis.ws.zbdbdot.DbStatus;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Date;
import java.util.Properties;

import com.cezar.cezargis.ws.zbdbdot.ExportStatus;
import com.cezar.cezargis.ws.zbdbdot.FileStatus;
import com.cezar.cezargis.ws.zbdbdot.ZbdBdotWebServiceException;
import com.cezar.cezargis.ws.zbdbdot.model.ExportModel;

public class DbService {

    public static String PG_DUMP = getProperty("PG_DUMP");
    public static String DB_HOST = getProperty("DB_HOST_DUMP");
    public static String DB_PORT = getProperty("DB_PORT_DUMP");
    public static String BD_BASE = getProperty("BD_BASE_DUMP");
    public static String DB_SCHEMA_DUMP = getProperty("DB_SCHEMA_DUMP");    
    public static String DB_USER = getProperty("DB_USER_DUMP");
    public static String DB_PASS = getProperty("DB_PASS_DUMP");
    public static String FILE_DUMP = getProperty("FILE_DUMP");
    public static String FILE_PROCES = getProperty("FILE_PROCES");
    /*LOG*/
    public static String DB_HOST_LOG = getProperty("DB_HOST_LOG");
    public static String DB_PORT_LOG = getProperty("DB_PORT_LOG");
    public static String BD_BASE_LOG = getProperty("BD_BASE_LOG");
    public static String DB_SCHEMA_LOG = getProperty("DB_SCHEMA_LOG");
    public static String DB_USER_LOG = getProperty("DB_USER_LOG");
    public static String DB_PASS_LOG = getProperty("DB_PASS_LOG");
	
    private static Connection getConnection() throws Exception {
	Class.forName("org.postgresql.Driver");
	// DB_URL = jdbc:postgresql://localhost:5432/cezargis
	String db_url = "jdbc:postgresql://" + DB_HOST_LOG + ":" + DB_PORT_LOG + "/"
		+ BD_BASE_LOG;
	Connection c = DriverManager.getConnection(db_url, DB_USER_LOG, DB_PASS_LOG);
	c.setAutoCommit(false);
	return c;
    }

    /*-------------------------------------------------------------------*/
    /* ZAPIS i UPDATE PLIKU */
    /*-------------------------------------------------------------------*/
    /**
     * 
     * @param nazwaPliku
     * @param startCoutTimeFile
     * @param fs - FileStatus
     * @param es - ExportStatus
     * @return 
     */
    public static int saveToDB(String nazwaPliku, Long startCoutTimeFile, FileStatus fs, ExportStatus es) throws ZbdBdotWebServiceException {
	Date date = new Date();
	java.sql.Date sqlDate = new java.sql.Date(date.getTime());
	String sql = "INSERT INTO "
		+ DB_SCHEMA_LOG
		+ ".zbd_bdot (zbdt_plik_path, zbdt_plik_data, zbdt_plik_start, zbdt_plik_status, zbdt_plik_exportstatus) VALUES (?,?,?,?,?) RETURNING *;";
	Connection conn = null;
	PreparedStatement ps = null;
	ResultSet rs = null;
	try {
	    conn = getConnection();
	    ps = conn.prepareStatement(sql);
	    ps.setString(1, nazwaPliku);
	    ps.setDate(2, sqlDate);
	    ps.setLong(3, startCoutTimeFile);
	    ps.setString(4, fs.name());
	    ps.setString(5, es.name());
	    rs = ps.executeQuery();
	    conn.commit();
	    rs.next();
	    int row = rs.getRow();
	    if (row == 0) {
		throw new ZbdBdotWebServiceException(DbStatus.BLADZAPISU);
	    } else {
		return rs.getInt("zbdt_id");
	    }
	} catch (Exception e) {
	    try {
		conn.rollback();
	    } catch (Exception ignored) {
	    }
	    throw new ZbdBdotWebServiceException(e);
	} finally {
	    try {
		ps.close();
		conn.close();
	    } catch (Exception ignored) {
	    }
	}
    }

    public static ExportModel getFileExportObject() throws ZbdBdotWebServiceException {
	final String fileStatus = FileStatus.AKTUALNY.value();
	String sql = "SELECT * FROM " + DB_SCHEMA_LOG + ".zbd_bdot where zbdt_plik_status = ?";
	Connection conn = null;
	PreparedStatement ps = null;
	ResultSet rs = null;
	try {
	    conn = getConnection();
	    ps = conn.prepareStatement(sql);
	    ps.setString(1, fileStatus);
	    rs = ps.executeQuery();
	    conn.commit();
	    ExportModel em = new ExportModel();
	    if (rs.next()) {
		em.setZbdtId(rs.getInt("zbdt_id"));
		em.setZbdtPlikPath(rs.getString("zbdt_plik_path"));
		em.setZbdtPlikData(rs.getDate("zbdt_plik_data"));
		em.setZbdtPlikStart(rs.getLong("zbdt_plik_start"));
		em.setZbdtPlikStatus(FileStatus.fromValue(rs.getString("zbdt_plik_status")));
		em.setZbdtPlikExportStatus(ExportStatus.fromValue(rs.getString("zbdt_plik_exportstatus")));
		em.setZbdtPlikRozmiar(rs.getLong("zbdt_plik_rozmiar"));
		em.setZbdtPlikSumakontrolna(rs.getString("zbdt_plik_sumakontrolna"));
	    } else {
		throw new ZbdBdotWebServiceException(DbStatus.BRAKDANYCH);
	    }
	    if (rs.next()) {
		throw new ZbdBdotWebServiceException(DbStatus.WIELDANYCH);
	    }
	    return em;
	} catch (ZbdBdotWebServiceException e) {
	    throw new ZbdBdotWebServiceException(e.getDbStatus());
	} catch (Exception e) {
	    try {
		conn.rollback();
	    } catch (Exception ignored) {
	    }
	    throw new ZbdBdotWebServiceException(e);
	} finally {
	    try {
		rs.close();
		ps.close();
		conn.close();
	    } catch (Exception ignored) {
	    }
	}
    }

    public static void updateFileExportObject(ExportModel em) throws ZbdBdotWebServiceException {
	String sql = "UPDATE " + DB_SCHEMA_LOG + ".zbd_bdot SET zbdt_plik_status = ?, zbdt_plik_exportstatus = ?, zbdt_plik_rozmiar = ?, zbdt_plik_sumakontrolna=? where zbdt_plik_status = ?";
	Connection conn = null;
	PreparedStatement ps = null;
	try {
	    conn = getConnection();
	    ps = conn.prepareStatement(sql);
	    ps.setString(1, em.getZbdtPlikStatus().name());
	    ps.setString(2, em.getZbdtPlikExportStatus().name());
	    ps.setLong(3, em.getZbdtPlikRozmiar());
	    ps.setString(4, em.getZbdtPlikSumakontrolna());
	    ps.setString(5, FileStatus.AKTUALNY.name());
	    ps.executeUpdate();
	    conn.commit();
	} catch (Exception e) {
	    try {
		conn.rollback();
	    } catch (Exception ignored) {
	    }
	    throw new ZbdBdotWebServiceException(e);
	} finally {
	    try {
		ps.close();
		conn.close();
	    } catch (Exception ignored) {
	    }
	}
    }

    public static void setNonActiveStatus() throws ZbdBdotWebServiceException {
	String sql = "UPDATE " + DB_SCHEMA_LOG + ".zbd_bdot SET zbdt_plik_status = ?";
	Connection conn = null;
	PreparedStatement ps = null;
	try {
	    conn = getConnection();
	    ps = conn.prepareStatement(sql);
	    ps.setString(1, FileStatus.NIEAKTUALNY.name());
	    ps.executeUpdate();
	    conn.commit();
	} catch (Exception e) {
	    try {
		conn.rollback();
	    } catch (Exception ignored) {
	    }
	    throw new ZbdBdotWebServiceException(e);
	} finally {
	    try {
		ps.close();
		conn.close();
	    } catch (Exception ignored) {
	    }
	}
    }
    /*-------------------------------------------------------------------*/
    /* ZAPIS DO LOGÓW */
    /*-------------------------------------------------------------------*/

    /**
     * 
     * @param Id - id pliku exportowanego
     * @param es ExportStatus
     */
    public static void setToLog(int Id, ExportStatus es) throws ZbdBdotWebServiceException {
	Date date = new Date();
	java.sql.Date sqlDate = new java.sql.Date(date.getTime());
	String sql = "INSERT INTO "
		+ DB_SCHEMA_LOG
		+ ".zbd_bdot_log (zbdtl_bdot_id, zbdtl_status_tworzenia, zbdtl_status_data) VALUES (?,?,?);";

	Connection conn = null;
	PreparedStatement ps = null;
	try {
	    conn = getConnection();
	    ps = conn.prepareStatement(sql);
	    ps.setInt(1, Id);
	    ps.setString(2, es.name());
	    ps.setDate(3, sqlDate);
	    ps.executeUpdate();
	    conn.commit();
	} catch (Exception e) {
	    try {
		conn.rollback();
	    } catch (Exception ignored) {
	    }
	    throw new ZbdBdotWebServiceException(e);
	} finally {
	    try {
		ps.close();
		conn.close();
	    } catch (Exception ignored) {
	    }
	}
    }

    public static void truncateSchema() throws ZbdBdotWebServiceException {
	try {
	    StringBuilder sb = new StringBuilder();
	    sb.append("SELECT " + DB_SCHEMA_LOG + ".zbdbdot_truncateschemazbdbdot()");
	    callBDReturnVoid(sb.toString());
	} catch (Exception e) {
	    throw new ZbdBdotWebServiceException(e);
	}
    }

    public static void callBDReturnVoid(String sql) throws ZbdBdotWebServiceException {
	Connection conn = null;
	PreparedStatement ps = null;
	try {
	    conn = getConnection();
	    ps = conn.prepareStatement(sql);
	    ps.execute();
	    conn.commit();
	} catch (Exception e) {
	    try {
		conn.rollback();
	    } catch (Exception ignored) {
	    }
	    throw new ZbdBdotWebServiceException(e);
	} finally {
	    try {
		ps.close();
		conn.close();
	    } catch (Exception ignored) {
	    }
	}
    }

    private static String getProperty(String property) {
	try {
	    Properties properties = new Properties();
	    properties.load(DbService.class.getResourceAsStream("db.properties"));
	    return properties.getProperty(property);
	} catch (Exception e) {
	    e.printStackTrace();
	    return "";
	}
    }
}
