package com.cezar.cezargis.ws.zbdbdot.service.impl;

import com.cezar.cezargis.ws.zbdbdot.Export;
import com.cezar.cezargis.ws.zbdbdot.ExportStatus;
import com.cezar.cezargis.ws.zbdbdot.ZbdBdotWebServiceException;
import com.cezar.cezargis.ws.zbdbdot.schema.FaultInfo;
import com.cezar.cezargis.ws.zbdbdot.schema.PobierzRozmiarPliku;
import com.cezar.cezargis.ws.zbdbdot.schema.PobierzRozmiarPlikuFault;
import com.cezar.cezargis.ws.zbdbdot.schema.PobierzRozmiarPlikuResponse;
import com.cezar.cezargis.ws.zbdbdot.schema.PotwierdzRozmiarPliku;
import com.cezar.cezargis.ws.zbdbdot.schema.PotwierdzRozmiarPlikuFault;
import com.cezar.cezargis.ws.zbdbdot.schema.PotwierdzRozmiarPlikuResponse;
import com.cezar.cezargis.ws.zbdbdot.schema.PrzygotujPlik;
import com.cezar.cezargis.ws.zbdbdot.schema.PrzygotujPlikFault;
import com.cezar.cezargis.ws.zbdbdot.schema.PrzygotujPlikResponse;
import com.cezar.cezargis.ws.zbdbdot.schema.StatusPrzygotowaniaType;
import com.cezar.cezargis.ws.zbdbdot.schema.WyslijPlikDumpa;
import com.cezar.cezargis.ws.zbdbdot.schema.WyslijPlikDumpaFault;
import com.cezar.cezargis.ws.zbdbdot.schema.WyslijPlikDumpaResponse;
import com.cezar.cezargis.ws.zbdbdot.schema.ZbdBdotService;
import javax.activation.DataHandler;
import javax.jws.WebService;
import javax.xml.ws.soap.MTOM;
import org.apache.log4j.Logger; 

@MTOM
@WebService(targetNamespace = "http://ws.cezargis.cezar.com/zbdbdot/1.0",
serviceName = "ZbdBdotServicePorts",
portName = "ZbdBdotService",
wsdlLocation = "/WEB-INF/wsdl/zbdbdot.wsdl",
endpointInterface = "com.cezar.cezargis.ws.zbdbdot.schema.ZbdBdotService")
public class ZbdBdotServiceImpl implements ZbdBdotService {

    private static final Logger logger = Logger.getLogger(ZbdBdotServiceImpl.class);

    @Override
    public PrzygotujPlikResponse przygotujPlik(PrzygotujPlik parameters) throws PrzygotujPlikFault {
	PrzygotujPlikResponse przygotujPlikResponse = new PrzygotujPlikResponse();
	przygotujPlikResponse.setStatusPrzygotowania(StatusPrzygotowaniaType.ROZPOCZECIE_GENEROWANY);
	try {
	    Export.runBackup();
	} catch (ZbdBdotWebServiceException e) {
	    przygotujPlikResponse.setStatusPrzygotowania(StatusPrzygotowaniaType.BLAD);
	    FaultInfo faultInfo = new FaultInfo();
	    if (e.getExportStatus() != null) {
		StatusPrzygotowaniaType statusType = getDumpStatus(e.getExportStatus());
		przygotujPlikResponse.setStatusPrzygotowania(statusType);
		faultInfo.setStatusPrzygotowania(statusType);
	    }
	    throw new PrzygotujPlikFault(e.getMessage(), faultInfo, e);
	}
	return przygotujPlikResponse;
    }

    @Override
    public PobierzRozmiarPlikuResponse pobierzRozmiarPliku(
	    PobierzRozmiarPliku parameters) throws PobierzRozmiarPlikuFault {
	PobierzRozmiarPlikuResponse pobierzRozmiarPlikuResponse = new PobierzRozmiarPlikuResponse();
	Long fileSizeFromDB = null;
	try {
	    fileSizeFromDB = Export.getFileSizeFromDB();
	} catch (ZbdBdotWebServiceException e) {
	    FaultInfo faultInfo = new FaultInfo();
	    if (e.getExportStatus() != null) {
		StatusPrzygotowaniaType statusType = getDumpStatus(e.getExportStatus());
		pobierzRozmiarPlikuResponse.setStatusPrzygotowania(statusType);
		faultInfo.setStatusPrzygotowania(statusType);
	    }
	    throw new PobierzRozmiarPlikuFault(e.getMessage(), faultInfo, e);
	}
	if (fileSizeFromDB != null) {
	    pobierzRozmiarPlikuResponse.setRozmiarPliku(fileSizeFromDB);
	    if (fileSizeFromDB == 0) {
		pobierzRozmiarPlikuResponse.setStatusPrzygotowania(StatusPrzygotowaniaType.GENEROWANY);
	    } else {
		pobierzRozmiarPlikuResponse.setStatusPrzygotowania(StatusPrzygotowaniaType.PLIK_OK);
	    }
	} else {
	    pobierzRozmiarPlikuResponse.setStatusPrzygotowania(StatusPrzygotowaniaType.PLIK_ERROR);
	}
	return pobierzRozmiarPlikuResponse;
    }

    @Override
    public WyslijPlikDumpaResponse wyslijPlikDumpa(WyslijPlikDumpa parameters)
	    throws WyslijPlikDumpaFault {

	WyslijPlikDumpaResponse wyslijPlikDumpaResponse = new WyslijPlikDumpaResponse();

	DataHandler dumpFile = null;
	try {
	    wyslijPlikDumpaResponse.setStatusPrzygotowania(StatusPrzygotowaniaType.ROZPOCZECIE_WYSYLANIA);
	    dumpFile = Export.getDumpFile();
	} catch (ZbdBdotWebServiceException e) {
	    FaultInfo faultInfo = new FaultInfo();
	    if (e.getExportStatus() != null) {
		StatusPrzygotowaniaType statusType = getDumpStatus(e.getExportStatus());
		wyslijPlikDumpaResponse.setStatusPrzygotowania(statusType);
		faultInfo.setStatusPrzygotowania(statusType);
	    }
	    throw new WyslijPlikDumpaFault(e.getMessage(), faultInfo, e);
	} finally {
	}

	wyslijPlikDumpaResponse.setLargeAmountOfData(dumpFile);
	return wyslijPlikDumpaResponse;
    }

    @Override
    public PotwierdzRozmiarPlikuResponse potwierdzRozmiarPliku(
	    PotwierdzRozmiarPliku parameters) throws PotwierdzRozmiarPlikuFault {

	PotwierdzRozmiarPlikuResponse potwierdzRozmiarPlikuResponse = new PotwierdzRozmiarPlikuResponse();
	long rozmiarPliku = parameters.getRozmiarPliku();
	String sumaKontrolna = parameters.getSumaKontrolna();

	try {
	    Export.checkFile(rozmiarPliku, sumaKontrolna);
	    potwierdzRozmiarPlikuResponse.setStatusPrzygotowania(StatusPrzygotowaniaType.PLIK_POPRAWNY);
	} catch (ZbdBdotWebServiceException e) {
	    potwierdzRozmiarPlikuResponse.setStatusPrzygotowania(StatusPrzygotowaniaType.PLIK_ERROR);
	    FaultInfo faultInfo = new FaultInfo();
	    if (e.getExportStatus() != null) {
		StatusPrzygotowaniaType statusType = getDumpStatus(e.getExportStatus());
		potwierdzRozmiarPlikuResponse.setStatusPrzygotowania(statusType);
		faultInfo.setStatusPrzygotowania(statusType);
	    }
	    throw new PotwierdzRozmiarPlikuFault(e.getMessage(), faultInfo, e);
	} finally {
	}
	return potwierdzRozmiarPlikuResponse;
    }

    private StatusPrzygotowaniaType getDumpStatus(ExportStatus es) {

	StatusPrzygotowaniaType value = StatusPrzygotowaniaType.BLAD;
	switch (es) {

	    case ROZPOCZECIE_GENEROWANY:
		value = StatusPrzygotowaniaType.ROZPOCZECIE_GENEROWANY;
		break;
	    case GENEROWANY:
		value = StatusPrzygotowaniaType.GENEROWANY;
		break;
	    case BLAD_GENEROWANIA:
		value = StatusPrzygotowaniaType.BLAD_GENEROWANIA;
		break;
	    case PLIK_OK:
		value = StatusPrzygotowaniaType.PLIK_OK;
		break;
	    case PLIK_ERROR:
		value = StatusPrzygotowaniaType.PLIK_ERROR;
		break;
	    case PLIK_POPRAWNY:
		value = StatusPrzygotowaniaType.PLIK_POPRAWNY;
		break;
	    case WYGENEROWANY:
		value = StatusPrzygotowaniaType.WYGENEROWANY;
		break;
	    case ROZPOCZECIE_WYSYLANIA:
		value = StatusPrzygotowaniaType.ROZPOCZECIE_WYSYLANIA;
		break;
	    case POTWIERDZENIE_ODBIORU:
		value = StatusPrzygotowaniaType.POTWIERDZENIE_ODBIORU;
		break;
	    case BLAD:
		value = StatusPrzygotowaniaType.BLAD;
		break;
	    default:
		value = StatusPrzygotowaniaType.BLAD;
		break;

	}

	return value;
    }
}
